clear()
//
//------------------------------------------------------
//
//------------------------------------------------------
// GA for Rastrigin
disp('GA for Rastrigin');
//
//-----------------------------------------------------
//
//-----------------------------------------------------
// selection: proportionate with ranking method 
// crossover: blend 
// mutation: non uniform
// 1-elitism
//
function y=rastrigin(x)  // the function to optimize
n=prod(size(x));
y=n+sum(x.^2-cos(2*%pi*x));
endfunction
//
//-----------------------------------------------------
function evalpop=evaluation(X);  // evaluation of the whole population
[Npo,n]=size(X);n=n-1;
evalpop=zeros(Npo,1);
for i=1:Npo;
 evalpop(i)=rastrigin(X(i,1:n));
end
endfunction
//-----------------------------------------------------
function [Xpar,bestpar,npos]=evalplus1elitism(Xpar,Xoldpar,Npop,gen) // evaluation plus 1-elitism
val=evaluation(Xpar);    
Xpar(:,n+1)=val;
[minval,index]=min(Xoldpar(:,n+1));  
bestpar=Xoldpar(index(1),:);
[minval,index]=min(val);

bestnewpop=Xpar(index(1),:);
npos=index(1);
if (bestnewpop($)>bestpar($))&(gen>1) then
    nrand=int(Npop*rand())+1;
    Xpar(nrand,:)=bestpar;
    npos=nrand;
else 
    bestpar=bestnewpop;
end
endfunction
//-----------------------------------------------------
function Xsel=rankselect(X)    // ranking selection
Xsel=X;
[Npop,n]=size(X);n=n-1;
[Fgsort,ind]=gsort(X(:,n+1));
X=X(ind,:);
p=1:Npop;roulette=cumsum(p)/sum(p);
for i=1:Npop;
  u=rand();
  index=find(roulette<u);Nselect=max(index)+1;
  Xsel(i,:)=X(Nselect,:);
end
endfunction
//-----------------------------------------------------
function xoff=blcross(xpar)    // blend crossover
[np,n]=size(xpar);n=n-1;
xoff=xpar;
for i=1:n
   u=rand();
   xoff(1,:)=u*xpar(1,:)+(1-u)*xpar(2,:);
   xoff(2,:)=u*xpar(2,:)+(1-u)*xpar(1,:);
end
endfunction
//-----------------------------------------------------//
function xoff=mutate(xpar,ngen,Ngen,xmin,xmax) // non uniform mutation
[np,n]=size(xpar);n=n-1;
b=5;
xoff=xpar;
for i=1:n
  u1=rand();u2=rand();
  if (u1<1/2) then
    xoff(1,i)=xpar(1,i)+(xmax(i)-xpar(i))*u2*(1-(ngen-1)/Ngen)^b;
   else
    xoff(1,i)=xpar(1,i)-(xpar(i)-xmin(i))*u2*(1-(ngen-1)/Ngen)^b;
   end;
end;
endfunction
//
//
//-----------------------------------------------------
//-----------------------------------------------------
//---------main program -------------------------------
// ----------------------------------------------------
//
n=evstr(x_dialog('number of variables','2'));   // number of variables of the optimization problem
Npop=evstr(x_dialog('population number','50'));     
Ngen=evstr(x_dialog('generation number','20')); 
pc=0.9;                       // crossover coefficient
pm=0.6;                       // mutation coefficient
//
// ----------------------------------------------------
//
xmin=-5*ones(1,n);
xmax=5*ones(1,n);
//
Xmin=ones(Npop,1)*xmin;
Xmax=ones(Npop,1)*xmax;
u=rand(Npop,n);
pop=Xmin+(Xmax-Xmin).*u;  // random initialisation of the population
Xpar=[pop,rand(Npop,1)];  Xoldpar=Xpar;
Xoff=Xpar;Xoldpar=Xpar;fmin=[];Ntot=[];peno=[];
mineval=0;newval=zeros(Npop,1);CVR=1;valgen=0;itgrad=0;itfct=0;
Nevalf=0;
Nevalg=0;
//
//-----------------------------------------------------
//-----------------------------------------------------
//
//     MAIN LOOP : BEGINNING
//
//
for gen=1:(Ngen-1)
Xpar=Xoff;
[Xpar,bestpar,npos]=evalplus1elitism(Xpar,Xoldpar,Npop,gen);  // evaluation plus 1-elitism
Nevalf=Nevalf+Npop;
fmin=[fmin,bestpar($)];
Ntot=[Ntot,Nevalf];
//
Xoldpar=Xpar;
Xpar=rankselect(Xpar);  // GA rank selection
for i=1:2:(Npop-1)
    u1=int(Npop*rand())+1;u2=int(Npop*rand())+1;u3=rand();
    xpar=[Xpar(u1,:);Xpar(u2,:)];Xoff(i:(i+1),:)=xpar;
    if (u3<pc) then
     xoff=blcross(xpar);   // GA crossover
     else
      xoff=xpar;
     end
     Xoff(i:(i+1),:)=xoff;
     for j=1:2
       u4=rand();
        if (u4<pm) then
          xoffl=mutate(xoff(j,:),gen,Ngen,xmin,xmax);   //GA mutation
          Xoff(i+j-1,:)=xoffl;
        end
    end   
end
end
//
//     MAIN LOOP : ENDING
//
//-----------------------------------------------------
//-----------------------------------------------------
//
[Xpar,bestpar,npos]=evalplus1elitism(Xoff,Xoldpar,Npop,Ngen);  // last evaluation plus 1-elitism
Nevalf=Nevalf+Npop;
Ntot=[Ntot,Nevalf];
fmin=[fmin,bestpar($)];
//
xset('window',0)
clf
plot2d(Ntot,fmin)
//
//
disp('minimum obtained:');disp(bestpar(1:n));
disp('corresponding value by f:');disp(bestpar($));
disp('evaluation number:');disp(Nevalf);
//

